//////////////////////////////////////////////////////////////////////////
//	Name	: MediaFoundationSingleton.cpp
//
//////////////////////////////////////////////////////////////////////////
#include "config.h" //This goes first to define all of the proper ENABLEs and things

#include "MediaFoundationSingleton.h"

#include "assert.h"
#include "renderer/MediaFoundation/MFVSGCCShim.h"
extern "C" {
#include "libshim/include/shim_win32.h"
}

MediaFoundationSingleton * g_MediaFoundationSingleton = NULL;

MediaFoundationSingleton * const MediaFoundationSingleton::instance() {
	if (!g_MediaFoundationSingleton)
		g_MediaFoundationSingleton = new MediaFoundationSingleton();

	return g_MediaFoundationSingleton;
}

MediaFoundationSingleton::MediaFoundationSingleton()
//	: m_pShim(NULL)
//	, m_shimRetainCount(0)
	: m_pMediaDLL(0) {
}

tMFVSGCCShim *MediaFoundationSingleton::retainMediaPlayerShim() {
	m_mutex.lock();

	// Already have a shim
	//if (m_pShim) {
	//	++m_shimRetainCount;
	//	m_mutex.unlock();
	//	return m_pShim;
	//}

	//Load the media DLL if needed
	if (!m_pMediaDLL) {
		const char *basedir = shim_GetBaseDir();
		wchar_t basedir_w[strlen(basedir) + 1];
		memset(basedir_w, 0, sizeof(basedir_w));
		mbstowcs(basedir_w, basedir, strlen(basedir));
		wchar_t DLLFilePath[MAX_PATH] = {0};
		swprintf(DLLFilePath, L"%s\\%s", basedir_w, L"assets\\MediaFramework.dll");
		if(LoadMediaDLL(DLLFilePath)) {
			m_mutex.unlock();
			return NULL;
		}
	}

//	assert(!m_pShim);

	//Find the function to get the media player
	tMFVSGCCShim * (*GetMediaPlayerShim)() = (tMFVSGCCShim * (*)())GetProcAddress(m_pMediaDLL, "GetMediaPlayerShim");
	if (!GetMediaPlayerShim) {
		FreeLibrary(m_pMediaDLL);
		m_pMediaDLL = 0;
		m_mutex.unlock();
		return NULL;
	}

	//Get the media player from the DLL
	tMFVSGCCShim * pShim = GetMediaPlayerShim();
	if (!pShim) {
		FreeLibrary(m_pMediaDLL);
		m_pMediaDLL = 0;
		m_mutex.unlock();
		return NULL;
	}

	//++m_shimRetainCount;
	m_mutex.unlock();
	return pShim;
}

void MediaFoundationSingleton::releaseMediaPlayerShim(tMFVSGCCShim * pShim) {
	m_mutex.lock();

	//if (m_shimRetainCount) {
	//	--m_shimRetainCount;
	//	if (0 == m_shimRetainCount)
			UnloadMediaDLL(pShim);
//	}

	m_mutex.unlock();
}

//////////////////////////////////////////////////////////////////////////
//	Name	: LoadMediaDLL
//////////////////////////////////////////////////////////////////////////
MPERROR MediaFoundationSingleton::LoadMediaDLL(const wchar_t * vDLLName)
{
	//Make sure we haven't already loaded the media DLL
	if (m_pMediaDLL) {
		assert(false && "Attempting to load media DLL when it is already loaded.");
		return MPERROR_SUCCESS;
	}

	//Load the media DLL
	m_pMediaDLL = LoadLibraryW(vDLLName);
	if (!m_pMediaDLL)
		return MPERROR_CANNOT_LOAD_DLL;

	//All went well, return success;
	return MPERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////////
//	Name	: UnloadMediaDLL
//////////////////////////////////////////////////////////////////////////
void MediaFoundationSingleton::UnloadMediaDLL(tMFVSGCCShim * pShim)
{
	//If the DLL isn't loaded, we're done
	if (!m_pMediaDLL)
		return;

//	assert(0 == m_shimRetainCount);

	//Free used memory
	void (*FreeMediaPlayerShim)(tMFVSGCCShim *) = (void (*)(tMFVSGCCShim *))GetProcAddress(m_pMediaDLL, "FreeMediaPlayerShim");
	if (FreeMediaPlayerShim)
		FreeMediaPlayerShim(pShim);
	pShim = NULL;

	//Close the DLL
	FreeLibrary(m_pMediaDLL);
	m_pMediaDLL = 0;
}
