/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*

	About Compositing:
	
	Webkit implements compositing of images following the operations established by Porter-Duff. They
	defined a number of operations, which map precisely to the ones offered by Webkit:

        CompositeClear,
        CompositeCopy,
        CompositeSourceOver,
        CompositeSourceIn,
        CompositeSourceOut,
        CompositeSourceAtop,
        CompositeDestinationOver,
        CompositeDestinationIn,
        CompositeDestinationOut,
        CompositeDestinationAtop,
        CompositeXOR,
		
        CompositePlusDarker,
        CompositePlusLighter

	CompositeSourceOver

	...is the default one and often referred to as a "standard blend" operation. In OpenGL and games in
	general the simplest blend operations is this:
	
	CD' = CS*AS + CD*(1-AS)
	
	This is -=NOT=- the operation referred to here, though!
	
	The operation is rather this:
	
	CD' = CS*AS + CD*AD*(1-AS)
	
	...which hence adds in the influence from the destination's alpha component! The new alpha is defined
	like so:
	
	AD' = AS + AD*(1-AS)
	
	The color blend function cannot be easily expressed in modern blending HW unfortunately. A common trick
	to circumvent this is to pre-multiply the respective alpha components in the buffers, so that the actual
	equation boils down to:
	
	CD' = CS + CD*(1-AS)
	
	Which can easily be expressed in HW. Note that the CD' value does INCLUDE the pre-multuplication of AD'!
	Hence the destination buffer ALWAYS is premultiplied! This nicely solves the issue of then using the
	destination as source again!
	
	Side effect:
	
	A buffer always needs to contain pre-multiplied data! This needs to be considered in a couple of situations!
	
	- Tint colors must be pre-multiplied
	- Colors for simple, color polygons likewise (after all: it's just a tinted, solid, white polygon - in a sense)
	- Texture data needs to be pre-multiplied (if it contains any alpha)
	- Even clearing a buffer needs some thought! After all: RGB needs to be premultiplied! Hence, if it's all transparent
	  at the start (likely), it needs to be 0,0,0,0 and NOT e.g. 1,1,1,0!

	Also see here:
	
	http://en.wikipedia.org/wiki/Alpha_compositing
	http://download.java.net/media/jogl/jogl-2.x-docs/com/sun/opengl/util/texture/Texture.html (somewhat related)
	
*/


