/*
 * Copyright (c) 2014 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "WiiUMediaPlayerSkin.h"

namespace WebCore {

static void convertARGBToRGBA(void* data, int width, int height) {
	int length = width * height * 4;
	unsigned char* charPtr = (unsigned char*)data;
	for (int i = 0; i < length; i += 4) {
		char tmp = (char) charPtr[i];
		charPtr[i] = charPtr[i+1];
		charPtr[i+1] = charPtr[i+2];
		charPtr[i+2] = charPtr[i+3];
		charPtr[i+3] = tmp;
	}
}

const unsigned char *WiiUMediaPlayerSkin::getBitmap(int index, int state, int *size)
{
	WiiUMediaSkinImage mediaSkinImage = cSkins[index];
	WiiUMediaSkinImageItem bitmap = mediaSkinImage.fImages[state];
	if(size)
		*size = bitmap.fSize;
	return bitmap.fBitmap;
}

void WiiUMediaPlayerSkin::getSize(int index, unsigned int* width, unsigned int* height)
{
	WiiUMediaSkinImage mediaSkinImage = cSkins[index];
	IntSize size = mediaSkinImage.size;
	*width = size.width();
	*height = size.height();
}

void WiiUMediaPlayerSkin::getPoints(int index, int state, IntPoint* points)
{
	WiiUMediaSkinImage mediaSkinImage = cSkins[index];
	WiiUMediaSkinImageItem bitmap = mediaSkinImage.fImages[state];
	memcpy(points, bitmap.fPoints, 4 * sizeof(IntPoint));
}

}


