/*
 * Copyright (C) 2010, Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ReverbNode_h
#define ReverbNode_h

#include "AudioGain.h"
#include "AudioNode.h"
#include "PlatformString.h"
#include <wtf/PassRefPtr.h>
#include <wtf/Threading.h>

namespace WebCore {

class AudioContext;


// ReverbNode is an AudioNode with one input and one output
// that controls the hardware Reverb effect.

class ReverbNode : public AudioNode {
public:
    static PassRefPtr<ReverbNode> create(AudioContext* context, double sampleRate)
    {
        return adoptRef(new ReverbNode(context, sampleRate));      
    }

    // AudioNode
    virtual void process(size_t framesToProcess);
    virtual void reset();

    // Called in the main thread when the number of channels for the input may have changed.
    virtual void checkNumberOfChannelsForInput(AudioNodeInput*);

    // JavaScript interface for attributes
	bool	highQuality()	const	{ return m_highQuality; }
	float	preDelay()		const	{ return m_preDelay; }
	float	reverbTime()	const	{ return m_reverbTime; }
	float	coloration()	const	{ return m_coloration; }
	float	damping()		const	{ return m_damping; }
	float	crosstalk()		const	{ return m_crosstalk; }
	float	mix()			const	{ return m_mix; }

	void setHighQuality( bool k )	{ m_highQuality = k; }
	void setPreDelay( float k )		{ m_preDelay	= k; }
	void setReverbTime( float k )	{ m_reverbTime	= k; }
	void setColoration( float k )	{ m_coloration	= k; }
	void setDamping( float k )		{ m_damping		= k; }
	void setCrosstalk( float k )	{ m_crosstalk	= k; }
	void setMix( float k )			{ m_mix			= k; }
    
private:
    ReverbNode(AudioContext*, double);

	bool		m_highQuality;
	double		m_preDelay;
	double		m_reverbTime;
	double		m_coloration;
	double		m_damping;
	double		m_crosstalk;
	double		m_mix;

    // This synchronizes live channel count changes which require an uninitialization / re-initialization.
    // FIXME: this can go away when we implement optimization for mixing with gain directly in summing junction of AudioNodeInput.
    mutable Mutex m_processLock;
};


} // namespace WebCore

#endif // ReverbNode_h
